﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.UI.DataVisualization.Charting;
using System.IO;

namespace MvcSparklines
{
    public class SparklineResult : FileResult
    {
        IEnumerable<float> samples;
        int SparklineWidth;
        int SparklineHeight;

        public SparklineResult(int width, int height, IEnumerable<float> samples) : base("image/png")
        {
            this.samples = samples;
            this.SparklineHeight = height;
            this.SparklineWidth = width;
        }

        protected override void WriteFile(HttpResponseBase response)
        {
            var sparklineImage = GetSparkLineImage(samples, SparklineWidth, SparklineHeight);
            response.ContentType = "image/png";
            response.BinaryWrite(sparklineImage);
        }

        private static byte[] GetSparkLineImage(IEnumerable<float> samples, int width, int height)
        {
            var chart = new Chart();

            var series = new Series();
            chart.Series.Add(series);
            // Sparklines use the 'Spline' chart type to show a smoother trend with a line chart
            series.ChartType = SeriesChartType.Spline;
            // Since the line is the only thing you see on the chart, you might want to
            // increase its width. Interestingly, you need to set the BorderWidth property
            // in order to accomplish that.
            series.BorderWidth = 2;
            // Add samples to the series
            int i = 0;
            foreach (var sample in samples)
            {
                i++;
                // Add 5 to the sample so line does not get cut off at the bottom of the image
                chart.Series[0].Points.AddXY(DateTime.Now.AddDays(i), sample + 5);
            }

            // Start hiding both sets of axes, labels, gridlines and tick marks
            var chartArea = new ChartArea();
            chart.ChartAreas.Add(chartArea);
            chartArea.AxisX.LabelStyle.Enabled = false;
            chartArea.AxisY.LabelStyle.Enabled = false;
            chartArea.AxisX.MajorGrid.Enabled = false;
            chartArea.AxisY.MajorGrid.Enabled = false;
            chartArea.AxisX.MajorTickMark.Enabled = false;
            chartArea.AxisY.MajorTickMark.Enabled = false;
            chartArea.AxisX.LineWidth = 0;
            chartArea.AxisY.LineWidth = 0;
            chartArea.AxisY.Minimum = 0;
            // Add an extra 5 pixels since the samples were adjusted up by 5
            // this prevents the line from being truncated
            chartArea.AxisY.Maximum = 105;

            // Re-adjust the size of the chart to reduce unnecessary white space
            chart.Width = width;
            chart.Height = height;

            // Get bytes of PNG image
            var s = new MemoryStream();
            chart.SaveImage(s, ChartImageFormat.Png);
            s.Position = 0;
            return s.ToArray();
        }

    }
}